from . import dream3dreview

def adaptive_alignment(data_container_array, global_correction = None, input_path = None, shift_x = None, shift_y = None, image_data_array_path = None, new_cell_array_name = None, min_radius = None, max_radius = None, number_circles = None, observer = None) -> int:
  """
  AdaptiveAlignment

  :param data_container_array | DataContainerArray
  :param global_correction | int GlobalCorrection
  :param input_path | QString InputPath
  :param shift_x | float ShiftX
  :param shift_y | float ShiftY
  :param image_data_array_path | DataArrayPath ImageDataArrayPath
  :param new_cell_array_name | QString NewCellArrayName
  :param min_radius | float MinRadius
  :param max_radius | float MaxRadius
  :param number_circles | int NumberCircles
  :param observer | Observer
  """

  filter = dream3dreview.AdaptiveAlignment()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if global_correction is not None:
    filter.GlobalCorrection = global_correction
  if input_path is not None:
    filter.InputPath = input_path
  if shift_x is not None:
    filter.ShiftX = shift_x
  if shift_y is not None:
    filter.ShiftY = shift_y
  if image_data_array_path is not None:
    filter.ImageDataArrayPath = image_data_array_path
  if new_cell_array_name is not None:
    filter.NewCellArrayName = new_cell_array_name
  if min_radius is not None:
    filter.MinRadius = min_radius
  if max_radius is not None:
    filter.MaxRadius = max_radius
  if number_circles is not None:
    filter.NumberCircles = number_circles
  filter.execute()
  return filter.ErrorCode

def apply_transformation_to_geometry(data_container_array, manual_transformation_matrix = None, computed_transformation_matrix = None, geometry_to_transform = None, transformation_matrix_type = None, rotation_axis = None, rotation_angle = None, translation = None, scale = None, observer = None) -> int:
  """
  ApplyTransformationToGeometry

  :param data_container_array | DataContainerArray
  :param manual_transformation_matrix | DynamicTableData ManualTransformationMatrix
  :param computed_transformation_matrix | DataArrayPath ComputedTransformationMatrix
  :param geometry_to_transform | DataArrayPath GeometryToTransform
  :param transformation_matrix_type | int TransformationMatrixType
  :param rotation_axis | FloatVec3Type RotationAxis
  :param rotation_angle | float RotationAngle
  :param translation | FloatVec3Type Translation
  :param scale | FloatVec3Type Scale
  :param observer | Observer
  """

  filter = dream3dreview.ApplyTransformationToGeometry()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if manual_transformation_matrix is not None:
    filter.ManualTransformationMatrix = manual_transformation_matrix
  if computed_transformation_matrix is not None:
    filter.ComputedTransformationMatrix = computed_transformation_matrix
  if geometry_to_transform is not None:
    filter.GeometryToTransform = geometry_to_transform
  if transformation_matrix_type is not None:
    filter.TransformationMatrixType = transformation_matrix_type
  if rotation_axis is not None:
    filter.RotationAxis = rotation_axis
  if rotation_angle is not None:
    filter.RotationAngle = rotation_angle
  if translation is not None:
    filter.Translation = translation
  if scale is not None:
    filter.Scale = scale
  filter.execute()
  return filter.ErrorCode

def average_edge_face_cell_array_to_vertex_array(data_container_array, selected_array_path = None, average_vertex_array_path = None, observer = None) -> int:
  """
  AverageEdgeFaceCellArrayToVertexArray

  :param data_container_array | DataContainerArray
  :param selected_array_path | DataArrayPath SelectedArrayPath
  :param average_vertex_array_path | DataArrayPath AverageVertexArrayPath
  :param observer | Observer
  """

  filter = dream3dreview.AverageEdgeFaceCellArrayToVertexArray()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if selected_array_path is not None:
    filter.SelectedArrayPath = selected_array_path
  if average_vertex_array_path is not None:
    filter.AverageVertexArrayPath = average_vertex_array_path
  filter.execute()
  return filter.ErrorCode

def average_vertex_array_to_edge_face_cell_array(data_container_array, selected_array_path = None, average_cell_array_path = None, weighted_average = None, observer = None) -> int:
  """
  AverageVertexArrayToEdgeFaceCellArray

  :param data_container_array | DataContainerArray
  :param selected_array_path | DataArrayPath SelectedArrayPath
  :param average_cell_array_path | DataArrayPath AverageCellArrayPath
  :param weighted_average | bool WeightedAverage
  :param observer | Observer
  """

  filter = dream3dreview.AverageVertexArrayToEdgeFaceCellArray()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if selected_array_path is not None:
    filter.SelectedArrayPath = selected_array_path
  if average_cell_array_path is not None:
    filter.AverageCellArrayPath = average_cell_array_path
  if weighted_average is not None:
    filter.WeightedAverage = weighted_average
  filter.execute()
  return filter.ErrorCode

def compute_umeyama_transform(data_container_array, source_point_set = None, dest_point_set = None, use_scaling = None, transformation_attribute_matrix_name = None, transformation_matrix_name = None, observer = None) -> int:
  """
  ComputeUmeyamaTransform

  :param data_container_array | DataContainerArray
  :param source_point_set | DataArrayPath SourcePointSet
  :param dest_point_set | DataArrayPath DestPointSet
  :param use_scaling | bool UseScaling
  :param transformation_attribute_matrix_name | QString TransformationAttributeMatrixName
  :param transformation_matrix_name | QString TransformationMatrixName
  :param observer | Observer
  """

  filter = dream3dreview.ComputeUmeyamaTransform()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if source_point_set is not None:
    filter.SourcePointSet = source_point_set
  if dest_point_set is not None:
    filter.DestPointSet = dest_point_set
  if use_scaling is not None:
    filter.UseScaling = use_scaling
  if transformation_attribute_matrix_name is not None:
    filter.TransformationAttributeMatrixName = transformation_attribute_matrix_name
  if transformation_matrix_name is not None:
    filter.TransformationMatrixName = transformation_matrix_name
  filter.execute()
  return filter.ErrorCode

def establish_foam_morphology(data_container_array, data_container_name = None, output_cell_attribute_matrix_path = None, input_cell_feature_ids_array_path = None, output_cell_ensemble_attribute_matrix_name = None, output_cell_feature_attribute_matrix_name = None, feature_ids_array_name = None, mask_array_name = None, cell_phases_array_name = None, num_features_array_name = None, qp_euclidean_distances_array_name = None, tj_euclidean_distances_array_name = None, gb_euclidean_distances_array_name = None, input_stats_array_path = None, input_phase_types_array_path = None, input_phase_names_array_path = None, input_shape_types_array_path = None, csv_output_file = None, periodic_boundaries = None, write_goal_attributes = None, have_features = None, min_strut_thickness = None, strut_thickness_variability = None, strut_shape_variability = None, smooth_struts = None, observer = None) -> int:
  """
  EstablishFoamMorphology

  :param data_container_array | DataContainerArray
  :param data_container_name | DataArrayPath DataContainerName
  :param output_cell_attribute_matrix_path | DataArrayPath OutputCellAttributeMatrixPath
  :param input_cell_feature_ids_array_path | DataArrayPath InputCellFeatureIdsArrayPath
  :param output_cell_ensemble_attribute_matrix_name | QString OutputCellEnsembleAttributeMatrixName
  :param output_cell_feature_attribute_matrix_name | QString OutputCellFeatureAttributeMatrixName
  :param feature_ids_array_name | QString FeatureIdsArrayName
  :param mask_array_name | QString MaskArrayName
  :param cell_phases_array_name | QString CellPhasesArrayName
  :param num_features_array_name | QString NumFeaturesArrayName
  :param qp_euclidean_distances_array_name | QString QPEuclideanDistancesArrayName
  :param tj_euclidean_distances_array_name | QString TJEuclideanDistancesArrayName
  :param gb_euclidean_distances_array_name | QString GBEuclideanDistancesArrayName
  :param input_stats_array_path | DataArrayPath InputStatsArrayPath
  :param input_phase_types_array_path | DataArrayPath InputPhaseTypesArrayPath
  :param input_phase_names_array_path | DataArrayPath InputPhaseNamesArrayPath
  :param input_shape_types_array_path | DataArrayPath InputShapeTypesArrayPath
  :param csv_output_file | QString CsvOutputFile
  :param periodic_boundaries | bool PeriodicBoundaries
  :param write_goal_attributes | bool WriteGoalAttributes
  :param have_features | int HaveFeatures
  :param min_strut_thickness | double MinStrutThickness
  :param strut_thickness_variability | float StrutThicknessVariability
  :param strut_shape_variability | float StrutShapeVariability
  :param smooth_struts | bool SmoothStruts
  :param observer | Observer
  """

  filter = dream3dreview.EstablishFoamMorphology()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if data_container_name is not None:
    filter.DataContainerName = data_container_name
  if output_cell_attribute_matrix_path is not None:
    filter.OutputCellAttributeMatrixPath = output_cell_attribute_matrix_path
  if input_cell_feature_ids_array_path is not None:
    filter.InputCellFeatureIdsArrayPath = input_cell_feature_ids_array_path
  if output_cell_ensemble_attribute_matrix_name is not None:
    filter.OutputCellEnsembleAttributeMatrixName = output_cell_ensemble_attribute_matrix_name
  if output_cell_feature_attribute_matrix_name is not None:
    filter.OutputCellFeatureAttributeMatrixName = output_cell_feature_attribute_matrix_name
  if feature_ids_array_name is not None:
    filter.FeatureIdsArrayName = feature_ids_array_name
  if mask_array_name is not None:
    filter.MaskArrayName = mask_array_name
  if cell_phases_array_name is not None:
    filter.CellPhasesArrayName = cell_phases_array_name
  if num_features_array_name is not None:
    filter.NumFeaturesArrayName = num_features_array_name
  if qp_euclidean_distances_array_name is not None:
    filter.QPEuclideanDistancesArrayName = qp_euclidean_distances_array_name
  if tj_euclidean_distances_array_name is not None:
    filter.TJEuclideanDistancesArrayName = tj_euclidean_distances_array_name
  if gb_euclidean_distances_array_name is not None:
    filter.GBEuclideanDistancesArrayName = gb_euclidean_distances_array_name
  if input_stats_array_path is not None:
    filter.InputStatsArrayPath = input_stats_array_path
  if input_phase_types_array_path is not None:
    filter.InputPhaseTypesArrayPath = input_phase_types_array_path
  if input_phase_names_array_path is not None:
    filter.InputPhaseNamesArrayPath = input_phase_names_array_path
  if input_shape_types_array_path is not None:
    filter.InputShapeTypesArrayPath = input_shape_types_array_path
  if csv_output_file is not None:
    filter.CsvOutputFile = csv_output_file
  if periodic_boundaries is not None:
    filter.PeriodicBoundaries = periodic_boundaries
  if write_goal_attributes is not None:
    filter.WriteGoalAttributes = write_goal_attributes
  if have_features is not None:
    filter.HaveFeatures = have_features
  if min_strut_thickness is not None:
    filter.MinStrutThickness = min_strut_thickness
  if strut_thickness_variability is not None:
    filter.StrutThicknessVariability = strut_thickness_variability
  if strut_shape_variability is not None:
    filter.StrutShapeVariability = strut_shape_variability
  if smooth_struts is not None:
    filter.SmoothStruts = smooth_struts
  filter.execute()
  return filter.ErrorCode

def dbscan(data_container_array, selected_array_path = None, use_mask = None, mask_array_path = None, feature_ids_array_name = None, feature_attribute_matrix_name = None, epsilon = None, min_pnts = None, distance_metric = None, observer = None) -> int:
  """
  DBSCAN

  :param data_container_array | DataContainerArray
  :param selected_array_path | DataArrayPath SelectedArrayPath
  :param use_mask | bool UseMask
  :param mask_array_path | DataArrayPath MaskArrayPath
  :param feature_ids_array_name | QString FeatureIdsArrayName
  :param feature_attribute_matrix_name | QString FeatureAttributeMatrixName
  :param epsilon | float Epsilon
  :param min_pnts | int MinPnts
  :param distance_metric | int DistanceMetric
  :param observer | Observer
  """

  filter = dream3dreview.DBSCAN()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if selected_array_path is not None:
    filter.SelectedArrayPath = selected_array_path
  if use_mask is not None:
    filter.UseMask = use_mask
  if mask_array_path is not None:
    filter.MaskArrayPath = mask_array_path
  if feature_ids_array_name is not None:
    filter.FeatureIdsArrayName = feature_ids_array_name
  if feature_attribute_matrix_name is not None:
    filter.FeatureAttributeMatrixName = feature_attribute_matrix_name
  if epsilon is not None:
    filter.Epsilon = epsilon
  if min_pnts is not None:
    filter.MinPnts = min_pnts
  if distance_metric is not None:
    filter.DistanceMetric = distance_metric
  filter.execute()
  return filter.ErrorCode

def downsample_vertex_geometry(data_container_array, vertex_attr_mat_path = None, downsample_type = None, decimation_freq = None, decimation_fraction = None, grid_resolution = None, observer = None) -> int:
  """
  DownsampleVertexGeometry

  :param data_container_array | DataContainerArray
  :param vertex_attr_mat_path | DataArrayPath VertexAttrMatPath
  :param downsample_type | int DownsampleType
  :param decimation_freq | int DecimationFreq
  :param decimation_fraction | float DecimationFraction
  :param grid_resolution | FloatVec3Type GridResolution
  :param observer | Observer
  """

  filter = dream3dreview.DownsampleVertexGeometry()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if vertex_attr_mat_path is not None:
    filter.VertexAttrMatPath = vertex_attr_mat_path
  if downsample_type is not None:
    filter.DownsampleType = downsample_type
  if decimation_freq is not None:
    filter.DecimationFreq = decimation_freq
  if decimation_fraction is not None:
    filter.DecimationFraction = decimation_fraction
  if grid_resolution is not None:
    filter.GridResolution = grid_resolution
  filter.execute()
  return filter.ErrorCode

def extract_internal_surfaces_from_triangle_geometry(data_container_array, triangle_data_container_name = None, node_types_array_path = None, internal_triangles_name = None, observer = None) -> int:
  """
  ExtractInternalSurfacesFromTriangleGeometry

  :param data_container_array | DataContainerArray
  :param triangle_data_container_name | DataArrayPath TriangleDataContainerName
  :param node_types_array_path | DataArrayPath NodeTypesArrayPath
  :param internal_triangles_name | QString InternalTrianglesName
  :param observer | Observer
  """

  filter = dream3dreview.ExtractInternalSurfacesFromTriangleGeometry()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if triangle_data_container_name is not None:
    filter.TriangleDataContainerName = triangle_data_container_name
  if node_types_array_path is not None:
    filter.NodeTypesArrayPath = node_types_array_path
  if internal_triangles_name is not None:
    filter.InternalTrianglesName = internal_triangles_name
  filter.execute()
  return filter.ErrorCode

def find_array_statistics(data_container_array, find_histogram = None, find_length = None, find_min = None, find_max = None, find_mean = None, find_median = None, find_std_deviation = None, find_summation = None, use_mask = None, standardize_data = None, compute_by_index = None, destination_attribute_matrix = None, mask_array_path = None, length_array_name = None, minimum_array_name = None, maximum_array_name = None, mean_array_name = None, median_array_name = None, std_deviation_array_name = None, summation_array_name = None, standardized_array_name = None, selected_array_path = None, feature_ids_array_path = None, histogram_array_name = None, use_full_range = None, num_bins = None, min_range = None, max_range = None, observer = None) -> int:
  """
  FindArrayStatistics

  :param data_container_array | DataContainerArray
  :param find_histogram | bool FindHistogram
  :param find_length | bool FindLength
  :param find_min | bool FindMin
  :param find_max | bool FindMax
  :param find_mean | bool FindMean
  :param find_median | bool FindMedian
  :param find_std_deviation | bool FindStdDeviation
  :param find_summation | bool FindSummation
  :param use_mask | bool UseMask
  :param standardize_data | bool StandardizeData
  :param compute_by_index | bool ComputeByIndex
  :param destination_attribute_matrix | DataArrayPath DestinationAttributeMatrix
  :param mask_array_path | DataArrayPath MaskArrayPath
  :param length_array_name | QString LengthArrayName
  :param minimum_array_name | QString MinimumArrayName
  :param maximum_array_name | QString MaximumArrayName
  :param mean_array_name | QString MeanArrayName
  :param median_array_name | QString MedianArrayName
  :param std_deviation_array_name | QString StdDeviationArrayName
  :param summation_array_name | QString SummationArrayName
  :param standardized_array_name | QString StandardizedArrayName
  :param selected_array_path | DataArrayPath SelectedArrayPath
  :param feature_ids_array_path | DataArrayPath FeatureIdsArrayPath
  :param histogram_array_name | QString HistogramArrayName
  :param use_full_range | bool UseFullRange
  :param num_bins | int32_t NumBins
  :param min_range | float MinRange
  :param max_range | float MaxRange
  :param observer | Observer
  """

  filter = dream3dreview.FindArrayStatistics()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if find_histogram is not None:
    filter.FindHistogram = find_histogram
  if find_length is not None:
    filter.FindLength = find_length
  if find_min is not None:
    filter.FindMin = find_min
  if find_max is not None:
    filter.FindMax = find_max
  if find_mean is not None:
    filter.FindMean = find_mean
  if find_median is not None:
    filter.FindMedian = find_median
  if find_std_deviation is not None:
    filter.FindStdDeviation = find_std_deviation
  if find_summation is not None:
    filter.FindSummation = find_summation
  if use_mask is not None:
    filter.UseMask = use_mask
  if standardize_data is not None:
    filter.StandardizeData = standardize_data
  if compute_by_index is not None:
    filter.ComputeByIndex = compute_by_index
  if destination_attribute_matrix is not None:
    filter.DestinationAttributeMatrix = destination_attribute_matrix
  if mask_array_path is not None:
    filter.MaskArrayPath = mask_array_path
  if length_array_name is not None:
    filter.LengthArrayName = length_array_name
  if minimum_array_name is not None:
    filter.MinimumArrayName = minimum_array_name
  if maximum_array_name is not None:
    filter.MaximumArrayName = maximum_array_name
  if mean_array_name is not None:
    filter.MeanArrayName = mean_array_name
  if median_array_name is not None:
    filter.MedianArrayName = median_array_name
  if std_deviation_array_name is not None:
    filter.StdDeviationArrayName = std_deviation_array_name
  if summation_array_name is not None:
    filter.SummationArrayName = summation_array_name
  if standardized_array_name is not None:
    filter.StandardizedArrayName = standardized_array_name
  if selected_array_path is not None:
    filter.SelectedArrayPath = selected_array_path
  if feature_ids_array_path is not None:
    filter.FeatureIdsArrayPath = feature_ids_array_path
  if histogram_array_name is not None:
    filter.HistogramArrayName = histogram_array_name
  if use_full_range is not None:
    filter.UseFullRange = use_full_range
  if num_bins is not None:
    filter.NumBins = num_bins
  if min_range is not None:
    filter.MinRange = min_range
  if max_range is not None:
    filter.MaxRange = max_range
  filter.execute()
  return filter.ErrorCode

def find_element_centroids(data_container_array, cell_centroids_array_path = None, create_vertex_data_container = None, new_data_container_name = None, vertex_attribute_matrix_name = None, observer = None) -> int:
  """
  FindElementCentroids

  :param data_container_array | DataContainerArray
  :param cell_centroids_array_path | DataArrayPath CellCentroidsArrayPath
  :param create_vertex_data_container | bool CreateVertexDataContainer
  :param new_data_container_name | DataArrayPath NewDataContainerName
  :param vertex_attribute_matrix_name | QString VertexAttributeMatrixName
  :param observer | Observer
  """

  filter = dream3dreview.FindElementCentroids()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if cell_centroids_array_path is not None:
    filter.CellCentroidsArrayPath = cell_centroids_array_path
  if create_vertex_data_container is not None:
    filter.CreateVertexDataContainer = create_vertex_data_container
  if new_data_container_name is not None:
    filter.NewDataContainerName = new_data_container_name
  if vertex_attribute_matrix_name is not None:
    filter.VertexAttributeMatrixName = vertex_attribute_matrix_name
  filter.execute()
  return filter.ErrorCode

def find_norm(data_container_array, selected_array_path = None, norm_array_path = None, p_space = None, observer = None) -> int:
  """
  FindNorm

  :param data_container_array | DataContainerArray
  :param selected_array_path | DataArrayPath SelectedArrayPath
  :param norm_array_path | DataArrayPath NormArrayPath
  :param p_space | float PSpace
  :param observer | Observer
  """

  filter = dream3dreview.FindNorm()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if selected_array_path is not None:
    filter.SelectedArrayPath = selected_array_path
  if norm_array_path is not None:
    filter.NormArrayPath = norm_array_path
  if p_space is not None:
    filter.PSpace = p_space
  filter.execute()
  return filter.ErrorCode

def find_vertex_to_triangle_distances(data_container_array, vertex_data_container = None, triangle_data_container = None, triangle_normals_array_path = None, distances_array_path = None, closest_triangle_id_array_path = None, observer = None) -> int:
  """
  FindVertexToTriangleDistances

  :param data_container_array | DataContainerArray
  :param vertex_data_container | DataArrayPath VertexDataContainer
  :param triangle_data_container | DataArrayPath TriangleDataContainer
  :param triangle_normals_array_path | DataArrayPath TriangleNormalsArrayPath
  :param distances_array_path | DataArrayPath DistancesArrayPath
  :param closest_triangle_id_array_path | DataArrayPath ClosestTriangleIdArrayPath
  :param observer | Observer
  """

  filter = dream3dreview.FindVertexToTriangleDistances()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if vertex_data_container is not None:
    filter.VertexDataContainer = vertex_data_container
  if triangle_data_container is not None:
    filter.TriangleDataContainer = triangle_data_container
  if triangle_normals_array_path is not None:
    filter.TriangleNormalsArrayPath = triangle_normals_array_path
  if distances_array_path is not None:
    filter.DistancesArrayPath = distances_array_path
  if closest_triangle_id_array_path is not None:
    filter.ClosestTriangleIdArrayPath = closest_triangle_id_array_path
  filter.execute()
  return filter.ErrorCode

def iterative_closest_point(data_container_array, moving_vertex_geometry = None, target_vertex_geometry = None, iterations = None, apply_transform = None, transform_attribute_matrix_name = None, transform_array_name = None, observer = None) -> int:
  """
  IterativeClosestPoint

  :param data_container_array | DataContainerArray
  :param moving_vertex_geometry | DataArrayPath MovingVertexGeometry
  :param target_vertex_geometry | DataArrayPath TargetVertexGeometry
  :param iterations | int Iterations
  :param apply_transform | bool ApplyTransform
  :param transform_attribute_matrix_name | QString TransformAttributeMatrixName
  :param transform_array_name | QString TransformArrayName
  :param observer | Observer
  """

  filter = dream3dreview.IterativeClosestPoint()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if moving_vertex_geometry is not None:
    filter.MovingVertexGeometry = moving_vertex_geometry
  if target_vertex_geometry is not None:
    filter.TargetVertexGeometry = target_vertex_geometry
  if iterations is not None:
    filter.Iterations = iterations
  if apply_transform is not None:
    filter.ApplyTransform = apply_transform
  if transform_attribute_matrix_name is not None:
    filter.TransformAttributeMatrixName = transform_attribute_matrix_name
  if transform_array_name is not None:
    filter.TransformArrayName = transform_array_name
  filter.execute()
  return filter.ErrorCode

def import_qm_meltpool_h5_file(data_container_array, input_files = None, data_container_path = None, vertex_attribute_matrix_name = None, slice_range = None, power = None, possible_indices = None, observer = None) -> int:
  """
  ImportQMMeltpoolH5File

  :param data_container_array | DataContainerArray
  :param input_files | VectString InputFiles
  :param data_container_path | DataArrayPath DataContainerPath
  :param vertex_attribute_matrix_name | QString VertexAttributeMatrixName
  :param slice_range | IntVec2Type SliceRange
  :param power | float Power
  :param possible_indices | QString PossibleIndices
  :param observer | Observer
  """

  filter = dream3dreview.ImportQMMeltpoolH5File()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if input_files is not None:
    filter.InputFiles = input_files
  if data_container_path is not None:
    filter.DataContainerPath = data_container_path
  if vertex_attribute_matrix_name is not None:
    filter.VertexAttributeMatrixName = vertex_attribute_matrix_name
  if slice_range is not None:
    filter.SliceRange = slice_range
  if power is not None:
    filter.Power = power
  if possible_indices is not None:
    filter.PossibleIndices = possible_indices
  filter.execute()
  return filter.ErrorCode

def k_distance_graph(data_container_array, selected_array_path = None, use_mask = None, mask_array_path = None, k_distance_array_path = None, min_dist = None, distance_metric = None, observer = None) -> int:
  """
  KDistanceGraph

  :param data_container_array | DataContainerArray
  :param selected_array_path | DataArrayPath SelectedArrayPath
  :param use_mask | bool UseMask
  :param mask_array_path | DataArrayPath MaskArrayPath
  :param k_distance_array_path | DataArrayPath KDistanceArrayPath
  :param min_dist | int MinDist
  :param distance_metric | int DistanceMetric
  :param observer | Observer
  """

  filter = dream3dreview.KDistanceGraph()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if selected_array_path is not None:
    filter.SelectedArrayPath = selected_array_path
  if use_mask is not None:
    filter.UseMask = use_mask
  if mask_array_path is not None:
    filter.MaskArrayPath = mask_array_path
  if k_distance_array_path is not None:
    filter.KDistanceArrayPath = k_distance_array_path
  if min_dist is not None:
    filter.MinDist = min_dist
  if distance_metric is not None:
    filter.DistanceMetric = distance_metric
  filter.execute()
  return filter.ErrorCode

def k_means(data_container_array, selected_array_path = None, use_mask = None, mask_array_path = None, feature_ids_array_name = None, means_array_name = None, init_clusters = None, feature_attribute_matrix_name = None, distance_metric = None, observer = None) -> int:
  """
  KMeans

  :param data_container_array | DataContainerArray
  :param selected_array_path | DataArrayPath SelectedArrayPath
  :param use_mask | bool UseMask
  :param mask_array_path | DataArrayPath MaskArrayPath
  :param feature_ids_array_name | QString FeatureIdsArrayName
  :param means_array_name | QString MeansArrayName
  :param init_clusters | int InitClusters
  :param feature_attribute_matrix_name | QString FeatureAttributeMatrixName
  :param distance_metric | int DistanceMetric
  :param observer | Observer
  """

  filter = dream3dreview.KMeans()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if selected_array_path is not None:
    filter.SelectedArrayPath = selected_array_path
  if use_mask is not None:
    filter.UseMask = use_mask
  if mask_array_path is not None:
    filter.MaskArrayPath = mask_array_path
  if feature_ids_array_name is not None:
    filter.FeatureIdsArrayName = feature_ids_array_name
  if means_array_name is not None:
    filter.MeansArrayName = means_array_name
  if init_clusters is not None:
    filter.InitClusters = init_clusters
  if feature_attribute_matrix_name is not None:
    filter.FeatureAttributeMatrixName = feature_attribute_matrix_name
  if distance_metric is not None:
    filter.DistanceMetric = distance_metric
  filter.execute()
  return filter.ErrorCode

def k_medoids(data_container_array, selected_array_path = None, use_mask = None, mask_array_path = None, feature_ids_array_name = None, medoids_array_name = None, feature_attribute_matrix_name = None, init_clusters = None, distance_metric = None, observer = None) -> int:
  """
  KMedoids

  :param data_container_array | DataContainerArray
  :param selected_array_path | DataArrayPath SelectedArrayPath
  :param use_mask | bool UseMask
  :param mask_array_path | DataArrayPath MaskArrayPath
  :param feature_ids_array_name | QString FeatureIdsArrayName
  :param medoids_array_name | QString MedoidsArrayName
  :param feature_attribute_matrix_name | QString FeatureAttributeMatrixName
  :param init_clusters | int InitClusters
  :param distance_metric | int DistanceMetric
  :param observer | Observer
  """

  filter = dream3dreview.KMedoids()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if selected_array_path is not None:
    filter.SelectedArrayPath = selected_array_path
  if use_mask is not None:
    filter.UseMask = use_mask
  if mask_array_path is not None:
    filter.MaskArrayPath = mask_array_path
  if feature_ids_array_name is not None:
    filter.FeatureIdsArrayName = feature_ids_array_name
  if medoids_array_name is not None:
    filter.MedoidsArrayName = medoids_array_name
  if feature_attribute_matrix_name is not None:
    filter.FeatureAttributeMatrixName = feature_attribute_matrix_name
  if init_clusters is not None:
    filter.InitClusters = init_clusters
  if distance_metric is not None:
    filter.DistanceMetric = distance_metric
  filter.execute()
  return filter.ErrorCode

def normalize_arrays(data_container_array, selected_data_array_paths = None, normalize_type = None, range_min = None, range_max = None, postfix = None, use_mask = None, mask_array_path = None, default_value = None, observer = None) -> int:
  """
  NormalizeArrays

  :param data_container_array | DataContainerArray
  :param selected_data_array_paths | std::vector<DataArrayPath> SelectedDataArrayPaths
  :param normalize_type | int NormalizeType
  :param range_min | double RangeMin
  :param range_max | double RangeMax
  :param postfix | QString Postfix
  :param use_mask | bool UseMask
  :param mask_array_path | DataArrayPath MaskArrayPath
  :param default_value | double DefaultValue
  :param observer | Observer
  """

  filter = dream3dreview.NormalizeArrays()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if selected_data_array_paths is not None:
    filter.SelectedDataArrayPaths = selected_data_array_paths
  if normalize_type is not None:
    filter.NormalizeType = normalize_type
  if range_min is not None:
    filter.RangeMin = range_min
  if range_max is not None:
    filter.RangeMax = range_max
  if postfix is not None:
    filter.Postfix = postfix
  if use_mask is not None:
    filter.UseMask = use_mask
  if mask_array_path is not None:
    filter.MaskArrayPath = mask_array_path
  if default_value is not None:
    filter.DefaultValue = default_value
  filter.execute()
  return filter.ErrorCode

def point_sample_triangle_geometry(data_container_array, samples_number_type = None, triangle_geometry = None, vertex_geometry = None, vertex_attribute_matrix_name = None, number_of_samples = None, parent_geometry = None, triangle_areas_array_path = None, use_mask = None, mask_array_path = None, selected_data_array_paths = None, observer = None) -> int:
  """
  PointSampleTriangleGeometry

  :param data_container_array | DataContainerArray
  :param samples_number_type | int SamplesNumberType
  :param triangle_geometry | DataArrayPath TriangleGeometry
  :param vertex_geometry | QString VertexGeometry
  :param vertex_attribute_matrix_name | QString VertexAttributeMatrixName
  :param number_of_samples | int NumberOfSamples
  :param parent_geometry | DataArrayPath ParentGeometry
  :param triangle_areas_array_path | DataArrayPath TriangleAreasArrayPath
  :param use_mask | bool UseMask
  :param mask_array_path | DataArrayPath MaskArrayPath
  :param selected_data_array_paths | std::vector<DataArrayPath> SelectedDataArrayPaths
  :param observer | Observer
  """

  filter = dream3dreview.PointSampleTriangleGeometry()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if samples_number_type is not None:
    filter.SamplesNumberType = samples_number_type
  if triangle_geometry is not None:
    filter.TriangleGeometry = triangle_geometry
  if vertex_geometry is not None:
    filter.VertexGeometry = vertex_geometry
  if vertex_attribute_matrix_name is not None:
    filter.VertexAttributeMatrixName = vertex_attribute_matrix_name
  if number_of_samples is not None:
    filter.NumberOfSamples = number_of_samples
  if parent_geometry is not None:
    filter.ParentGeometry = parent_geometry
  if triangle_areas_array_path is not None:
    filter.TriangleAreasArrayPath = triangle_areas_array_path
  if use_mask is not None:
    filter.UseMask = use_mask
  if mask_array_path is not None:
    filter.MaskArrayPath = mask_array_path
  if selected_data_array_paths is not None:
    filter.SelectedDataArrayPaths = selected_data_array_paths
  filter.execute()
  return filter.ErrorCode

def potts_model(data_container_array, iterations = None, temperature = None, periodic_boundaries = None, use_mask = None, feature_ids_array_path = None, mask_array_path = None, observer = None) -> int:
  """
  PottsModel

  :param data_container_array | DataContainerArray
  :param iterations | int Iterations
  :param temperature | double Temperature
  :param periodic_boundaries | bool PeriodicBoundaries
  :param use_mask | bool UseMask
  :param feature_ids_array_path | DataArrayPath FeatureIdsArrayPath
  :param mask_array_path | DataArrayPath MaskArrayPath
  :param observer | Observer
  """

  filter = dream3dreview.PottsModel()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if iterations is not None:
    filter.Iterations = iterations
  if temperature is not None:
    filter.Temperature = temperature
  if periodic_boundaries is not None:
    filter.PeriodicBoundaries = periodic_boundaries
  if use_mask is not None:
    filter.UseMask = use_mask
  if feature_ids_array_path is not None:
    filter.FeatureIdsArrayPath = feature_ids_array_path
  if mask_array_path is not None:
    filter.MaskArrayPath = mask_array_path
  filter.execute()
  return filter.ErrorCode

def principal_component_analysis(data_container_array, selected_data_array_paths = None, pc_attribute_matrix_name = None, pc_eigenvalues_name = None, pc_eigenvectors_name = None, matrix_approach = None, project_data_space = None, number_of_dimensions_for_projection = None, projected_data_space_array_path = None, observer = None) -> int:
  """
  PrincipalComponentAnalysis

  :param data_container_array | DataContainerArray
  :param selected_data_array_paths | std::vector<DataArrayPath> SelectedDataArrayPaths
  :param pc_attribute_matrix_name | QString PCAttributeMatrixName
  :param pc_eigenvalues_name | QString PCEigenvaluesName
  :param pc_eigenvectors_name | QString PCEigenvectorsName
  :param matrix_approach | int MatrixApproach
  :param project_data_space | bool ProjectDataSpace
  :param number_of_dimensions_for_projection | int NumberOfDimensionsForProjection
  :param projected_data_space_array_path | DataArrayPath ProjectedDataSpaceArrayPath
  :param observer | Observer
  """

  filter = dream3dreview.PrincipalComponentAnalysis()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if selected_data_array_paths is not None:
    filter.SelectedDataArrayPaths = selected_data_array_paths
  if pc_attribute_matrix_name is not None:
    filter.PCAttributeMatrixName = pc_attribute_matrix_name
  if pc_eigenvalues_name is not None:
    filter.PCEigenvaluesName = pc_eigenvalues_name
  if pc_eigenvectors_name is not None:
    filter.PCEigenvectorsName = pc_eigenvectors_name
  if matrix_approach is not None:
    filter.MatrixApproach = matrix_approach
  if project_data_space is not None:
    filter.ProjectDataSpace = project_data_space
  if number_of_dimensions_for_projection is not None:
    filter.NumberOfDimensionsForProjection = number_of_dimensions_for_projection
  if projected_data_space_array_path is not None:
    filter.ProjectedDataSpaceArrayPath = projected_data_space_array_path
  filter.execute()
  return filter.ErrorCode

def remove_flagged_vertices(data_container_array, vertex_geometry = None, mask_array_path = None, reduced_vertex_geometry = None, observer = None) -> int:
  """
  RemoveFlaggedVertices

  :param data_container_array | DataContainerArray
  :param vertex_geometry | DataArrayPath VertexGeometry
  :param mask_array_path | DataArrayPath MaskArrayPath
  :param reduced_vertex_geometry | QString ReducedVertexGeometry
  :param observer | Observer
  """

  filter = dream3dreview.RemoveFlaggedVertices()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if vertex_geometry is not None:
    filter.VertexGeometry = vertex_geometry
  if mask_array_path is not None:
    filter.MaskArrayPath = mask_array_path
  if reduced_vertex_geometry is not None:
    filter.ReducedVertexGeometry = reduced_vertex_geometry
  filter.execute()
  return filter.ErrorCode

def silhouette(data_container_array, selected_array_path = None, use_mask = None, mask_array_path = None, feature_ids_array_path = None, silhouette_array_path = None, distance_metric = None, observer = None) -> int:
  """
  Silhouette

  :param data_container_array | DataContainerArray
  :param selected_array_path | DataArrayPath SelectedArrayPath
  :param use_mask | bool UseMask
  :param mask_array_path | DataArrayPath MaskArrayPath
  :param feature_ids_array_path | DataArrayPath FeatureIdsArrayPath
  :param silhouette_array_path | DataArrayPath SilhouetteArrayPath
  :param distance_metric | int DistanceMetric
  :param observer | Observer
  """

  filter = dream3dreview.Silhouette()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if selected_array_path is not None:
    filter.SelectedArrayPath = selected_array_path
  if use_mask is not None:
    filter.UseMask = use_mask
  if mask_array_path is not None:
    filter.MaskArrayPath = mask_array_path
  if feature_ids_array_path is not None:
    filter.FeatureIdsArrayPath = feature_ids_array_path
  if silhouette_array_path is not None:
    filter.SilhouetteArrayPath = silhouette_array_path
  if distance_metric is not None:
    filter.DistanceMetric = distance_metric
  filter.execute()
  return filter.ErrorCode

def wave_front_object_file_writer(data_container_array, output_wave_front_file = None, triangle_geometry = None, observer = None) -> int:
  """
  WaveFrontObjectFileWriter

  :param data_container_array | DataContainerArray
  :param output_wave_front_file | QString OutputWaveFrontFile
  :param triangle_geometry | DataArrayPath TriangleGeometry
  :param observer | Observer
  """

  filter = dream3dreview.WaveFrontObjectFileWriter()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if output_wave_front_file is not None:
    filter.OutputWaveFrontFile = output_wave_front_file
  if triangle_geometry is not None:
    filter.TriangleGeometry = triangle_geometry
  filter.execute()
  return filter.ErrorCode

def robust_automatic_threshold(data_container_array, input_array_path = None, feature_ids_array_path = None, gradient_magnitude_array_path = None, observer = None) -> int:
  """
  RobustAutomaticThreshold

  :param data_container_array | DataContainerArray
  :param input_array_path | DataArrayPath InputArrayPath
  :param feature_ids_array_path | DataArrayPath FeatureIdsArrayPath
  :param gradient_magnitude_array_path | DataArrayPath GradientMagnitudeArrayPath
  :param observer | Observer
  """

  filter = dream3dreview.RobustAutomaticThreshold()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if input_array_path is not None:
    filter.InputArrayPath = input_array_path
  if feature_ids_array_path is not None:
    filter.FeatureIdsArrayPath = feature_ids_array_path
  if gradient_magnitude_array_path is not None:
    filter.GradientMagnitudeArrayPath = gradient_magnitude_array_path
  filter.execute()
  return filter.ErrorCode

def align_geometries(data_container_array, moving_geometry = None, target_geometry = None, alignment_type = None, observer = None) -> int:
  """
  AlignGeometries

  :param data_container_array | DataContainerArray
  :param moving_geometry | DataArrayPath MovingGeometry
  :param target_geometry | DataArrayPath TargetGeometry
  :param alignment_type | int AlignmentType
  :param observer | Observer
  """

  filter = dream3dreview.AlignGeometries()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if moving_geometry is not None:
    filter.MovingGeometry = moving_geometry
  if target_geometry is not None:
    filter.TargetGeometry = target_geometry
  if alignment_type is not None:
    filter.AlignmentType = alignment_type
  filter.execute()
  return filter.ErrorCode

def find_neighbor_list_statistics(data_container_array, find_length = None, find_min = None, find_max = None, find_mean = None, find_median = None, find_std_deviation = None, find_summation = None, destination_attribute_matrix = None, length_array_name = None, minimum_array_name = None, maximum_array_name = None, mean_array_name = None, median_array_name = None, std_deviation_array_name = None, summation_array_name = None, selected_array_path = None, observer = None) -> int:
  """
  FindNeighborListStatistics

  :param data_container_array | DataContainerArray
  :param find_length | bool FindLength
  :param find_min | bool FindMin
  :param find_max | bool FindMax
  :param find_mean | bool FindMean
  :param find_median | bool FindMedian
  :param find_std_deviation | bool FindStdDeviation
  :param find_summation | bool FindSummation
  :param destination_attribute_matrix | DataArrayPath DestinationAttributeMatrix
  :param length_array_name | QString LengthArrayName
  :param minimum_array_name | QString MinimumArrayName
  :param maximum_array_name | QString MaximumArrayName
  :param mean_array_name | QString MeanArrayName
  :param median_array_name | QString MedianArrayName
  :param std_deviation_array_name | QString StdDeviationArrayName
  :param summation_array_name | QString SummationArrayName
  :param selected_array_path | DataArrayPath SelectedArrayPath
  :param observer | Observer
  """

  filter = dream3dreview.FindNeighborListStatistics()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if find_length is not None:
    filter.FindLength = find_length
  if find_min is not None:
    filter.FindMin = find_min
  if find_max is not None:
    filter.FindMax = find_max
  if find_mean is not None:
    filter.FindMean = find_mean
  if find_median is not None:
    filter.FindMedian = find_median
  if find_std_deviation is not None:
    filter.FindStdDeviation = find_std_deviation
  if find_summation is not None:
    filter.FindSummation = find_summation
  if destination_attribute_matrix is not None:
    filter.DestinationAttributeMatrix = destination_attribute_matrix
  if length_array_name is not None:
    filter.LengthArrayName = length_array_name
  if minimum_array_name is not None:
    filter.MinimumArrayName = minimum_array_name
  if maximum_array_name is not None:
    filter.MaximumArrayName = maximum_array_name
  if mean_array_name is not None:
    filter.MeanArrayName = mean_array_name
  if median_array_name is not None:
    filter.MedianArrayName = median_array_name
  if std_deviation_array_name is not None:
    filter.StdDeviationArrayName = std_deviation_array_name
  if summation_array_name is not None:
    filter.SummationArrayName = summation_array_name
  if selected_array_path is not None:
    filter.SelectedArrayPath = selected_array_path
  filter.execute()
  return filter.ErrorCode

def interpolate_point_cloud_to_regular_grid(data_container_array, data_container_name = None, arrays_to_interpolate = None, arrays_to_copy = None, voxel_indices_array_path = None, interpolated_data_container_name = None, interpolated_attribute_matrix_name = None, interpolation_technique = None, kernel_size = None, sigmas = None, use_mask = None, mask_array_path = None, store_kernel_distances = None, kernel_distances_array_name = None, interpolated_suffix = None, copy_suffix = None, observer = None) -> int:
  """
  InterpolatePointCloudToRegularGrid

  :param data_container_array | DataContainerArray
  :param data_container_name | DataArrayPath DataContainerName
  :param arrays_to_interpolate | std::vector<DataArrayPath> ArraysToInterpolate
  :param arrays_to_copy | std::vector<DataArrayPath> ArraysToCopy
  :param voxel_indices_array_path | DataArrayPath VoxelIndicesArrayPath
  :param interpolated_data_container_name | DataArrayPath InterpolatedDataContainerName
  :param interpolated_attribute_matrix_name | QString InterpolatedAttributeMatrixName
  :param interpolation_technique | int InterpolationTechnique
  :param kernel_size | FloatVec3Type KernelSize
  :param sigmas | FloatVec3Type Sigmas
  :param use_mask | bool UseMask
  :param mask_array_path | DataArrayPath MaskArrayPath
  :param store_kernel_distances | bool StoreKernelDistances
  :param kernel_distances_array_name | QString KernelDistancesArrayName
  :param interpolated_suffix | QString InterpolatedSuffix
  :param copy_suffix | QString CopySuffix
  :param observer | Observer
  """

  filter = dream3dreview.InterpolatePointCloudToRegularGrid()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if data_container_name is not None:
    filter.DataContainerName = data_container_name
  if arrays_to_interpolate is not None:
    filter.ArraysToInterpolate = arrays_to_interpolate
  if arrays_to_copy is not None:
    filter.ArraysToCopy = arrays_to_copy
  if voxel_indices_array_path is not None:
    filter.VoxelIndicesArrayPath = voxel_indices_array_path
  if interpolated_data_container_name is not None:
    filter.InterpolatedDataContainerName = interpolated_data_container_name
  if interpolated_attribute_matrix_name is not None:
    filter.InterpolatedAttributeMatrixName = interpolated_attribute_matrix_name
  if interpolation_technique is not None:
    filter.InterpolationTechnique = interpolation_technique
  if kernel_size is not None:
    filter.KernelSize = kernel_size
  if sigmas is not None:
    filter.Sigmas = sigmas
  if use_mask is not None:
    filter.UseMask = use_mask
  if mask_array_path is not None:
    filter.MaskArrayPath = mask_array_path
  if store_kernel_distances is not None:
    filter.StoreKernelDistances = store_kernel_distances
  if kernel_distances_array_name is not None:
    filter.KernelDistancesArrayName = kernel_distances_array_name
  if interpolated_suffix is not None:
    filter.InterpolatedSuffix = interpolated_suffix
  if copy_suffix is not None:
    filter.CopySuffix = copy_suffix
  filter.execute()
  return filter.ErrorCode

def map_point_cloud_to_regular_grid(data_container_array, data_container_name = None, created_image_data_container_name = None, image_data_container_path = None, voxel_indices_array_path = None, grid_dimensions = None, use_mask = None, sampling_grid_type = None, mask_array_path = None, observer = None) -> int:
  """
  MapPointCloudToRegularGrid

  :param data_container_array | DataContainerArray
  :param data_container_name | DataArrayPath DataContainerName
  :param created_image_data_container_name | DataArrayPath CreatedImageDataContainerName
  :param image_data_container_path | DataArrayPath ImageDataContainerPath
  :param voxel_indices_array_path | DataArrayPath VoxelIndicesArrayPath
  :param grid_dimensions | IntVec3Type GridDimensions
  :param use_mask | bool UseMask
  :param sampling_grid_type | int SamplingGridType
  :param mask_array_path | DataArrayPath MaskArrayPath
  :param observer | Observer
  """

  filter = dream3dreview.MapPointCloudToRegularGrid()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if data_container_name is not None:
    filter.DataContainerName = data_container_name
  if created_image_data_container_name is not None:
    filter.CreatedImageDataContainerName = created_image_data_container_name
  if image_data_container_path is not None:
    filter.ImageDataContainerPath = image_data_container_path
  if voxel_indices_array_path is not None:
    filter.VoxelIndicesArrayPath = voxel_indices_array_path
  if grid_dimensions is not None:
    filter.GridDimensions = grid_dimensions
  if use_mask is not None:
    filter.UseMask = use_mask
  if sampling_grid_type is not None:
    filter.SamplingGridType = sampling_grid_type
  if mask_array_path is not None:
    filter.MaskArrayPath = mask_array_path
  filter.execute()
  return filter.ErrorCode

def read_binary_ct_north_star(data_container_array, input_header_file = None, data_container_name = None, cell_attribute_matrix_name = None, density_array_name = None, length_unit = None, import_subvolume = None, start_voxel_coord = None, end_voxel_coord = None, volume_description = None, data_file_info = None, imported_volume_description = None, observer = None) -> int:
  """
  ReadBinaryCTNorthStar

  :param data_container_array | DataContainerArray
  :param input_header_file | QString InputHeaderFile
  :param data_container_name | QString DataContainerName
  :param cell_attribute_matrix_name | QString CellAttributeMatrixName
  :param density_array_name | QString DensityArrayName
  :param length_unit | int32_t LengthUnit
  :param import_subvolume | bool ImportSubvolume
  :param start_voxel_coord | IntVec3Type StartVoxelCoord
  :param end_voxel_coord | IntVec3Type EndVoxelCoord
  :param volume_description | QString VolumeDescription
  :param data_file_info | QString DataFileInfo
  :param imported_volume_description | QString ImportedVolumeDescription
  :param observer | Observer
  """

  filter = dream3dreview.ReadBinaryCTNorthStar()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if input_header_file is not None:
    filter.InputHeaderFile = input_header_file
  if data_container_name is not None:
    filter.DataContainerName = data_container_name
  if cell_attribute_matrix_name is not None:
    filter.CellAttributeMatrixName = cell_attribute_matrix_name
  if density_array_name is not None:
    filter.DensityArrayName = density_array_name
  if length_unit is not None:
    filter.LengthUnit = length_unit
  if import_subvolume is not None:
    filter.ImportSubvolume = import_subvolume
  if start_voxel_coord is not None:
    filter.StartVoxelCoord = start_voxel_coord
  if end_voxel_coord is not None:
    filter.EndVoxelCoord = end_voxel_coord
  if volume_description is not None:
    filter.VolumeDescription = volume_description
  if data_file_info is not None:
    filter.DataFileInfo = data_file_info
  if imported_volume_description is not None:
    filter.ImportedVolumeDescription = imported_volume_description
  filter.execute()
  return filter.ErrorCode

def slice_triangle_geometry(data_container_array, cad_data_container_name = None, slice_data_container_name = None, edge_attribute_matrix_name = None, slice_attribute_matrix_name = None, slice_id_array_name = None, have_region_ids = None, region_id_array_path = None, slice_direction = None, slice_resolution = None, zstart = None, zend = None, slice_range = None, observer = None) -> int:
  """
  SliceTriangleGeometry

  :param data_container_array | DataContainerArray
  :param cad_data_container_name | DataArrayPath CADDataContainerName
  :param slice_data_container_name | QString SliceDataContainerName
  :param edge_attribute_matrix_name | QString EdgeAttributeMatrixName
  :param slice_attribute_matrix_name | QString SliceAttributeMatrixName
  :param slice_id_array_name | QString SliceIdArrayName
  :param have_region_ids | bool HaveRegionIds
  :param region_id_array_path | DataArrayPath RegionIdArrayPath
  :param slice_direction | FloatVec3Type SliceDirection
  :param slice_resolution | float SliceResolution
  :param zstart | float Zstart
  :param zend | float Zend
  :param slice_range | int SliceRange
  :param observer | Observer
  """

  filter = dream3dreview.SliceTriangleGeometry()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if cad_data_container_name is not None:
    filter.CADDataContainerName = cad_data_container_name
  if slice_data_container_name is not None:
    filter.SliceDataContainerName = slice_data_container_name
  if edge_attribute_matrix_name is not None:
    filter.EdgeAttributeMatrixName = edge_attribute_matrix_name
  if slice_attribute_matrix_name is not None:
    filter.SliceAttributeMatrixName = slice_attribute_matrix_name
  if slice_id_array_name is not None:
    filter.SliceIdArrayName = slice_id_array_name
  if have_region_ids is not None:
    filter.HaveRegionIds = have_region_ids
  if region_id_array_path is not None:
    filter.RegionIdArrayPath = region_id_array_path
  if slice_direction is not None:
    filter.SliceDirection = slice_direction
  if slice_resolution is not None:
    filter.SliceResolution = slice_resolution
  if zstart is not None:
    filter.Zstart = zstart
  if zend is not None:
    filter.Zend = zend
  if slice_range is not None:
    filter.SliceRange = slice_range
  filter.execute()
  return filter.ErrorCode

def steiner_compact(data_container_array, vtk_output = None, vtk_file_name = None, txt_output = None, txt_file_name = None, feature_ids_array_path = None, cell_phases_array_path = None, plane = None, sites = None, observer = None) -> int:
  """
  SteinerCompact

  :param data_container_array | DataContainerArray
  :param vtk_output | bool VtkOutput
  :param vtk_file_name | QString VtkFileName
  :param txt_output | bool TxtOutput
  :param txt_file_name | QString TxtFileName
  :param feature_ids_array_path | DataArrayPath FeatureIdsArrayPath
  :param cell_phases_array_path | DataArrayPath CellPhasesArrayPath
  :param plane | int Plane
  :param sites | int Sites
  :param observer | Observer
  """

  filter = dream3dreview.SteinerCompact()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if vtk_output is not None:
    filter.VtkOutput = vtk_output
  if vtk_file_name is not None:
    filter.VtkFileName = vtk_file_name
  if txt_output is not None:
    filter.TxtOutput = txt_output
  if txt_file_name is not None:
    filter.TxtFileName = txt_file_name
  if feature_ids_array_path is not None:
    filter.FeatureIdsArrayPath = feature_ids_array_path
  if cell_phases_array_path is not None:
    filter.CellPhasesArrayPath = cell_phases_array_path
  if plane is not None:
    filter.Plane = plane
  if sites is not None:
    filter.Sites = sites
  filter.execute()
  return filter.ErrorCode

def read_mic_data(data_container_array, data_container_name = None, cell_ensemble_attribute_matrix_name = None, cell_attribute_matrix_name = None, file_was_read = None, input_file = None, observer = None) -> int:
  """
  ReadMicData

  :param data_container_array | DataContainerArray
  :param data_container_name | DataArrayPath DataContainerName
  :param cell_ensemble_attribute_matrix_name | QString CellEnsembleAttributeMatrixName
  :param cell_attribute_matrix_name | QString CellAttributeMatrixName
  :param file_was_read | bool FileWasRead
  :param input_file | QString InputFile
  :param observer | Observer
  """

  filter = dream3dreview.ReadMicData()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if data_container_name is not None:
    filter.DataContainerName = data_container_name
  if cell_ensemble_attribute_matrix_name is not None:
    filter.CellEnsembleAttributeMatrixName = cell_ensemble_attribute_matrix_name
  if cell_attribute_matrix_name is not None:
    filter.CellAttributeMatrixName = cell_attribute_matrix_name
  if file_was_read is not None:
    filter.FileWasRead = file_was_read
  if input_file is not None:
    filter.InputFile = input_file
  filter.execute()
  return filter.ErrorCode

def tesselate_far_field_grains(data_container_array, output_cell_attribute_matrix_name = None, feature_ids_array_name = None, cell_phases_array_name = None, slab_id_array_name = None, feature_phases_array_name = None, feature_euler_angles_array_name = None, elastic_strains_array_name = None, centroids_array_name = None, volumes_array_name = None, axis_lengths_array_name = None, axis_euler_angles_array_name = None, omega3s_array_name = None, equivalent_diameters_array_name = None, crystal_structures_array_name = None, mask_array_path = None, feature_input_file_list_info = None, observer = None) -> int:
  """
  TesselateFarFieldGrains

  :param data_container_array | DataContainerArray
  :param output_cell_attribute_matrix_name | DataArrayPath OutputCellAttributeMatrixName
  :param feature_ids_array_name | QString FeatureIdsArrayName
  :param cell_phases_array_name | QString CellPhasesArrayName
  :param slab_id_array_name | QString SlabIdArrayName
  :param feature_phases_array_name | QString FeaturePhasesArrayName
  :param feature_euler_angles_array_name | QString FeatureEulerAnglesArrayName
  :param elastic_strains_array_name | QString ElasticStrainsArrayName
  :param centroids_array_name | QString CentroidsArrayName
  :param volumes_array_name | QString VolumesArrayName
  :param axis_lengths_array_name | QString AxisLengthsArrayName
  :param axis_euler_angles_array_name | QString AxisEulerAnglesArrayName
  :param omega3s_array_name | QString Omega3sArrayName
  :param equivalent_diameters_array_name | QString EquivalentDiametersArrayName
  :param crystal_structures_array_name | QString CrystalStructuresArrayName
  :param mask_array_path | DataArrayPath MaskArrayPath
  :param feature_input_file_list_info | StackFileListInfo FeatureInputFileListInfo
  :param observer | Observer
  """

  filter = dream3dreview.TesselateFarFieldGrains()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if output_cell_attribute_matrix_name is not None:
    filter.OutputCellAttributeMatrixName = output_cell_attribute_matrix_name
  if feature_ids_array_name is not None:
    filter.FeatureIdsArrayName = feature_ids_array_name
  if cell_phases_array_name is not None:
    filter.CellPhasesArrayName = cell_phases_array_name
  if slab_id_array_name is not None:
    filter.SlabIdArrayName = slab_id_array_name
  if feature_phases_array_name is not None:
    filter.FeaturePhasesArrayName = feature_phases_array_name
  if feature_euler_angles_array_name is not None:
    filter.FeatureEulerAnglesArrayName = feature_euler_angles_array_name
  if elastic_strains_array_name is not None:
    filter.ElasticStrainsArrayName = elastic_strains_array_name
  if centroids_array_name is not None:
    filter.CentroidsArrayName = centroids_array_name
  if volumes_array_name is not None:
    filter.VolumesArrayName = volumes_array_name
  if axis_lengths_array_name is not None:
    filter.AxisLengthsArrayName = axis_lengths_array_name
  if axis_euler_angles_array_name is not None:
    filter.AxisEulerAnglesArrayName = axis_euler_angles_array_name
  if omega3s_array_name is not None:
    filter.Omega3sArrayName = omega3s_array_name
  if equivalent_diameters_array_name is not None:
    filter.EquivalentDiametersArrayName = equivalent_diameters_array_name
  if crystal_structures_array_name is not None:
    filter.CrystalStructuresArrayName = crystal_structures_array_name
  if mask_array_path is not None:
    filter.MaskArrayPath = mask_array_path
  if feature_input_file_list_info is not None:
    filter.FeatureInputFileListInfo = feature_input_file_list_info
  filter.execute()
  return filter.ErrorCode

def ffthdf_writer_filter(data_container_array, output_file = None, write_eigenstrains = None, eigenstrains_output_file = None, feature_ids_array_path = None, cell_phases_array_path = None, cell_euler_angles_array_path = None, cell_eigenstrains_array_path = None, observer = None) -> int:
  """
  FFTHDFWriterFilter

  :param data_container_array | DataContainerArray
  :param output_file | QString OutputFile
  :param write_eigenstrains | bool WriteEigenstrains
  :param eigenstrains_output_file | QString EigenstrainsOutputFile
  :param feature_ids_array_path | DataArrayPath FeatureIdsArrayPath
  :param cell_phases_array_path | DataArrayPath CellPhasesArrayPath
  :param cell_euler_angles_array_path | DataArrayPath CellEulerAnglesArrayPath
  :param cell_eigenstrains_array_path | DataArrayPath CellEigenstrainsArrayPath
  :param observer | Observer
  """

  filter = dream3dreview.FFTHDFWriterFilter()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if output_file is not None:
    filter.OutputFile = output_file
  if write_eigenstrains is not None:
    filter.WriteEigenstrains = write_eigenstrains
  if eigenstrains_output_file is not None:
    filter.EigenstrainsOutputFile = eigenstrains_output_file
  if feature_ids_array_path is not None:
    filter.FeatureIdsArrayPath = feature_ids_array_path
  if cell_phases_array_path is not None:
    filter.CellPhasesArrayPath = cell_phases_array_path
  if cell_euler_angles_array_path is not None:
    filter.CellEulerAnglesArrayPath = cell_euler_angles_array_path
  if cell_eigenstrains_array_path is not None:
    filter.CellEigenstrainsArrayPath = cell_eigenstrains_array_path
  filter.execute()
  return filter.ErrorCode

def import_massif_data(data_container_array, massif_input_file_path = None, file_prefix = None, step_number = None, observer = None) -> int:
  """
  ImportMASSIFData

  :param data_container_array | DataContainerArray
  :param massif_input_file_path | QString MassifInputFilePath
  :param file_prefix | QString FilePrefix
  :param step_number | int StepNumber
  :param observer | Observer
  """

  filter = dream3dreview.ImportMASSIFData()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if massif_input_file_path is not None:
    filter.MassifInputFilePath = massif_input_file_path
  if file_prefix is not None:
    filter.FilePrefix = file_prefix
  if step_number is not None:
    filter.StepNumber = step_number
  filter.execute()
  return filter.ErrorCode

def local_dislocation_density_calculator(data_container_array, edge_data_container_name = None, burgers_vectors_array_path = None, slip_plane_normals_array_path = None, cell_size = None, output_data_container_name = None, output_attribute_matrix_name = None, output_array_name = None, dominant_system_array_name = None, observer = None) -> int:
  """
  LocalDislocationDensityCalculator

  :param data_container_array | DataContainerArray
  :param edge_data_container_name | DataArrayPath EdgeDataContainerName
  :param burgers_vectors_array_path | DataArrayPath BurgersVectorsArrayPath
  :param slip_plane_normals_array_path | DataArrayPath SlipPlaneNormalsArrayPath
  :param cell_size | FloatVec3Type CellSize
  :param output_data_container_name | DataArrayPath OutputDataContainerName
  :param output_attribute_matrix_name | QString OutputAttributeMatrixName
  :param output_array_name | QString OutputArrayName
  :param dominant_system_array_name | QString DominantSystemArrayName
  :param observer | Observer
  """

  filter = dream3dreview.LocalDislocationDensityCalculator()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if edge_data_container_name is not None:
    filter.EdgeDataContainerName = edge_data_container_name
  if burgers_vectors_array_path is not None:
    filter.BurgersVectorsArrayPath = burgers_vectors_array_path
  if slip_plane_normals_array_path is not None:
    filter.SlipPlaneNormalsArrayPath = slip_plane_normals_array_path
  if cell_size is not None:
    filter.CellSize = cell_size
  if output_data_container_name is not None:
    filter.OutputDataContainerName = output_data_container_name
  if output_attribute_matrix_name is not None:
    filter.OutputAttributeMatrixName = output_attribute_matrix_name
  if output_array_name is not None:
    filter.OutputArrayName = output_array_name
  if dominant_system_array_name is not None:
    filter.DominantSystemArrayName = dominant_system_array_name
  filter.execute()
  return filter.ErrorCode

def identify_dislocation_segments(data_container_array, burgers_vectors_array_path = None, slip_plane_normals_array_path = None, dislocation_ids_array_name = None, active_array_name = None, observer = None) -> int:
  """
  IdentifyDislocationSegments

  :param data_container_array | DataContainerArray
  :param burgers_vectors_array_path | DataArrayPath BurgersVectorsArrayPath
  :param slip_plane_normals_array_path | DataArrayPath SlipPlaneNormalsArrayPath
  :param dislocation_ids_array_name | QString DislocationIdsArrayName
  :param active_array_name | QString ActiveArrayName
  :param observer | Observer
  """

  filter = dream3dreview.IdentifyDislocationSegments()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if burgers_vectors_array_path is not None:
    filter.BurgersVectorsArrayPath = burgers_vectors_array_path
  if slip_plane_normals_array_path is not None:
    filter.SlipPlaneNormalsArrayPath = slip_plane_normals_array_path
  if dislocation_ids_array_name is not None:
    filter.DislocationIdsArrayName = dislocation_ids_array_name
  if active_array_name is not None:
    filter.ActiveArrayName = active_array_name
  filter.execute()
  return filter.ErrorCode

def discretize_dd_domain(data_container_array, edge_data_container_name = None, cell_size = None, output_data_container_name = None, output_attribute_matrix_name = None, output_array_name = None, observer = None) -> int:
  """
  DiscretizeDDDomain

  :param data_container_array | DataContainerArray
  :param edge_data_container_name | DataArrayPath EdgeDataContainerName
  :param cell_size | FloatVec3Type CellSize
  :param output_data_container_name | DataArrayPath OutputDataContainerName
  :param output_attribute_matrix_name | QString OutputAttributeMatrixName
  :param output_array_name | QString OutputArrayName
  :param observer | Observer
  """

  filter = dream3dreview.DiscretizeDDDomain()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if edge_data_container_name is not None:
    filter.EdgeDataContainerName = edge_data_container_name
  if cell_size is not None:
    filter.CellSize = cell_size
  if output_data_container_name is not None:
    filter.OutputDataContainerName = output_data_container_name
  if output_attribute_matrix_name is not None:
    filter.OutputAttributeMatrixName = output_attribute_matrix_name
  if output_array_name is not None:
    filter.OutputArrayName = output_array_name
  filter.execute()
  return filter.ErrorCode

def para_dis_reader(data_container_array, edge_data_container_name = None, vertex_attribute_matrix_name = None, edge_attribute_matrix_name = None, input_file = None, burgers_vector = None, number_of_arms_array_name = None, node_constraints_array_name = None, burgers_vectors_array_name = None, slip_plane_normals_array_name = None, domain_bounds_array_name = None, observer = None) -> int:
  """
  ParaDisReader

  :param data_container_array | DataContainerArray
  :param edge_data_container_name | DataArrayPath EdgeDataContainerName
  :param vertex_attribute_matrix_name | QString VertexAttributeMatrixName
  :param edge_attribute_matrix_name | QString EdgeAttributeMatrixName
  :param input_file | QString InputFile
  :param burgers_vector | float BurgersVector
  :param number_of_arms_array_name | QString NumberOfArmsArrayName
  :param node_constraints_array_name | QString NodeConstraintsArrayName
  :param burgers_vectors_array_name | QString BurgersVectorsArrayName
  :param slip_plane_normals_array_name | QString SlipPlaneNormalsArrayName
  :param domain_bounds_array_name | QString DomainBoundsArrayName
  :param observer | Observer
  """

  filter = dream3dreview.ParaDisReader()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if edge_data_container_name is not None:
    filter.EdgeDataContainerName = edge_data_container_name
  if vertex_attribute_matrix_name is not None:
    filter.VertexAttributeMatrixName = vertex_attribute_matrix_name
  if edge_attribute_matrix_name is not None:
    filter.EdgeAttributeMatrixName = edge_attribute_matrix_name
  if input_file is not None:
    filter.InputFile = input_file
  if burgers_vector is not None:
    filter.BurgersVector = burgers_vector
  if number_of_arms_array_name is not None:
    filter.NumberOfArmsArrayName = number_of_arms_array_name
  if node_constraints_array_name is not None:
    filter.NodeConstraintsArrayName = node_constraints_array_name
  if burgers_vectors_array_name is not None:
    filter.BurgersVectorsArrayName = burgers_vectors_array_name
  if slip_plane_normals_array_name is not None:
    filter.SlipPlaneNormalsArrayName = slip_plane_normals_array_name
  if domain_bounds_array_name is not None:
    filter.DomainBoundsArrayName = domain_bounds_array_name
  filter.execute()
  return filter.ErrorCode

def find_csl_boundaries(data_container_array, csl = None, axis_tolerance = None, angle_tolerance = None, avg_quats_array_path = None, feature_phases_array_path = None, crystal_structures_array_path = None, surface_mesh_face_labels_array_path = None, surface_mesh_face_normals_array_path = None, surface_mesh_csl_boundary_array_name = None, surface_mesh_csl_boundary_incoherence_array_name = None, observer = None) -> int:
  """
  FindCSLBoundaries

  :param data_container_array | DataContainerArray
  :param csl | float CSL
  :param axis_tolerance | float AxisTolerance
  :param angle_tolerance | float AngleTolerance
  :param avg_quats_array_path | DataArrayPath AvgQuatsArrayPath
  :param feature_phases_array_path | DataArrayPath FeaturePhasesArrayPath
  :param crystal_structures_array_path | DataArrayPath CrystalStructuresArrayPath
  :param surface_mesh_face_labels_array_path | DataArrayPath SurfaceMeshFaceLabelsArrayPath
  :param surface_mesh_face_normals_array_path | DataArrayPath SurfaceMeshFaceNormalsArrayPath
  :param surface_mesh_csl_boundary_array_name | QString SurfaceMeshCSLBoundaryArrayName
  :param surface_mesh_csl_boundary_incoherence_array_name | QString SurfaceMeshCSLBoundaryIncoherenceArrayName
  :param observer | Observer
  """

  filter = dream3dreview.FindCSLBoundaries()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if csl is not None:
    filter.CSL = csl
  if axis_tolerance is not None:
    filter.AxisTolerance = axis_tolerance
  if angle_tolerance is not None:
    filter.AngleTolerance = angle_tolerance
  if avg_quats_array_path is not None:
    filter.AvgQuatsArrayPath = avg_quats_array_path
  if feature_phases_array_path is not None:
    filter.FeaturePhasesArrayPath = feature_phases_array_path
  if crystal_structures_array_path is not None:
    filter.CrystalStructuresArrayPath = crystal_structures_array_path
  if surface_mesh_face_labels_array_path is not None:
    filter.SurfaceMeshFaceLabelsArrayPath = surface_mesh_face_labels_array_path
  if surface_mesh_face_normals_array_path is not None:
    filter.SurfaceMeshFaceNormalsArrayPath = surface_mesh_face_normals_array_path
  if surface_mesh_csl_boundary_array_name is not None:
    filter.SurfaceMeshCSLBoundaryArrayName = surface_mesh_csl_boundary_array_name
  if surface_mesh_csl_boundary_incoherence_array_name is not None:
    filter.SurfaceMeshCSLBoundaryIncoherenceArrayName = surface_mesh_csl_boundary_incoherence_array_name
  filter.execute()
  return filter.ErrorCode

def insert_transformation_phases(data_container_array, parent_phase = None, trans_crystal_struct = None, transformation_phase_misorientation = None, transformation_phase_habit_plane = None, define_habit_plane = None, use_all_variants = None, coherent_frac = None, transformation_phase_thickness = None, num_transformation_phases_per_feature = None, peninsula_frac = None, stats_gen_cell_ensemble_attribute_matrix_path = None, cell_feature_attribute_matrix_name = None, feature_ids_array_path = None, cell_euler_angles_array_path = None, cell_phases_array_path = None, avg_quats_array_path = None, centroids_array_path = None, equivalent_diameters_array_path = None, feature_euler_angles_array_path = None, feature_phases_array_path = None, feature_parent_ids_array_name = None, num_features_per_parent_array_path = None, crystal_structures_array_path = None, phase_types_array_path = None, shape_types_array_path = None, num_features_array_path = None, observer = None) -> int:
  """
  InsertTransformationPhases

  :param data_container_array | DataContainerArray
  :param parent_phase | int ParentPhase
  :param trans_crystal_struct | int TransCrystalStruct
  :param transformation_phase_misorientation | float TransformationPhaseMisorientation
  :param transformation_phase_habit_plane | FloatVec3Type TransformationPhaseHabitPlane
  :param define_habit_plane | bool DefineHabitPlane
  :param use_all_variants | bool UseAllVariants
  :param coherent_frac | float CoherentFrac
  :param transformation_phase_thickness | float TransformationPhaseThickness
  :param num_transformation_phases_per_feature | int NumTransformationPhasesPerFeature
  :param peninsula_frac | float PeninsulaFrac
  :param stats_gen_cell_ensemble_attribute_matrix_path | DataArrayPath StatsGenCellEnsembleAttributeMatrixPath
  :param cell_feature_attribute_matrix_name | DataArrayPath CellFeatureAttributeMatrixName
  :param feature_ids_array_path | DataArrayPath FeatureIdsArrayPath
  :param cell_euler_angles_array_path | DataArrayPath CellEulerAnglesArrayPath
  :param cell_phases_array_path | DataArrayPath CellPhasesArrayPath
  :param avg_quats_array_path | DataArrayPath AvgQuatsArrayPath
  :param centroids_array_path | DataArrayPath CentroidsArrayPath
  :param equivalent_diameters_array_path | DataArrayPath EquivalentDiametersArrayPath
  :param feature_euler_angles_array_path | DataArrayPath FeatureEulerAnglesArrayPath
  :param feature_phases_array_path | DataArrayPath FeaturePhasesArrayPath
  :param feature_parent_ids_array_name | QString FeatureParentIdsArrayName
  :param num_features_per_parent_array_path | DataArrayPath NumFeaturesPerParentArrayPath
  :param crystal_structures_array_path | DataArrayPath CrystalStructuresArrayPath
  :param phase_types_array_path | DataArrayPath PhaseTypesArrayPath
  :param shape_types_array_path | DataArrayPath ShapeTypesArrayPath
  :param num_features_array_path | DataArrayPath NumFeaturesArrayPath
  :param observer | Observer
  """

  filter = dream3dreview.InsertTransformationPhases()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if parent_phase is not None:
    filter.ParentPhase = parent_phase
  if trans_crystal_struct is not None:
    filter.TransCrystalStruct = trans_crystal_struct
  if transformation_phase_misorientation is not None:
    filter.TransformationPhaseMisorientation = transformation_phase_misorientation
  if transformation_phase_habit_plane is not None:
    filter.TransformationPhaseHabitPlane = transformation_phase_habit_plane
  if define_habit_plane is not None:
    filter.DefineHabitPlane = define_habit_plane
  if use_all_variants is not None:
    filter.UseAllVariants = use_all_variants
  if coherent_frac is not None:
    filter.CoherentFrac = coherent_frac
  if transformation_phase_thickness is not None:
    filter.TransformationPhaseThickness = transformation_phase_thickness
  if num_transformation_phases_per_feature is not None:
    filter.NumTransformationPhasesPerFeature = num_transformation_phases_per_feature
  if peninsula_frac is not None:
    filter.PeninsulaFrac = peninsula_frac
  if stats_gen_cell_ensemble_attribute_matrix_path is not None:
    filter.StatsGenCellEnsembleAttributeMatrixPath = stats_gen_cell_ensemble_attribute_matrix_path
  if cell_feature_attribute_matrix_name is not None:
    filter.CellFeatureAttributeMatrixName = cell_feature_attribute_matrix_name
  if feature_ids_array_path is not None:
    filter.FeatureIdsArrayPath = feature_ids_array_path
  if cell_euler_angles_array_path is not None:
    filter.CellEulerAnglesArrayPath = cell_euler_angles_array_path
  if cell_phases_array_path is not None:
    filter.CellPhasesArrayPath = cell_phases_array_path
  if avg_quats_array_path is not None:
    filter.AvgQuatsArrayPath = avg_quats_array_path
  if centroids_array_path is not None:
    filter.CentroidsArrayPath = centroids_array_path
  if equivalent_diameters_array_path is not None:
    filter.EquivalentDiametersArrayPath = equivalent_diameters_array_path
  if feature_euler_angles_array_path is not None:
    filter.FeatureEulerAnglesArrayPath = feature_euler_angles_array_path
  if feature_phases_array_path is not None:
    filter.FeaturePhasesArrayPath = feature_phases_array_path
  if feature_parent_ids_array_name is not None:
    filter.FeatureParentIdsArrayName = feature_parent_ids_array_name
  if num_features_per_parent_array_path is not None:
    filter.NumFeaturesPerParentArrayPath = num_features_per_parent_array_path
  if crystal_structures_array_path is not None:
    filter.CrystalStructuresArrayPath = crystal_structures_array_path
  if phase_types_array_path is not None:
    filter.PhaseTypesArrayPath = phase_types_array_path
  if shape_types_array_path is not None:
    filter.ShapeTypesArrayPath = shape_types_array_path
  if num_features_array_path is not None:
    filter.NumFeaturesArrayPath = num_features_array_path
  filter.execute()
  return filter.ErrorCode

def generate_feature_i_dsby_bounding_boxes(data_container_array, feature_i_ds_array_path = None, feature_attribute_matrix_array_path = None, box_center_array_path = None, box_dimensions_array_path = None, box_feature_i_ds_array_path = None, observer = None) -> int:
  """
  GenerateFeatureIDsbyBoundingBoxes

  :param data_container_array | DataContainerArray
  :param feature_i_ds_array_path | DataArrayPath FeatureIDsArrayPath
  :param feature_attribute_matrix_array_path | DataArrayPath FeatureAttributeMatrixArrayPath
  :param box_center_array_path | DataArrayPath BoxCenterArrayPath
  :param box_dimensions_array_path | DataArrayPath BoxDimensionsArrayPath
  :param box_feature_i_ds_array_path | DataArrayPath BoxFeatureIDsArrayPath
  :param observer | Observer
  """

  filter = dream3dreview.GenerateFeatureIDsbyBoundingBoxes()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if feature_i_ds_array_path is not None:
    filter.FeatureIDsArrayPath = feature_i_ds_array_path
  if feature_attribute_matrix_array_path is not None:
    filter.FeatureAttributeMatrixArrayPath = feature_attribute_matrix_array_path
  if box_center_array_path is not None:
    filter.BoxCenterArrayPath = box_center_array_path
  if box_dimensions_array_path is not None:
    filter.BoxDimensionsArrayPath = box_dimensions_array_path
  if box_feature_i_ds_array_path is not None:
    filter.BoxFeatureIDsArrayPath = box_feature_i_ds_array_path
  filter.execute()
  return filter.ErrorCode

def generate_mask_from_simple_shapes(data_container_array, mask_array_path = None, centers_array_path = None, axes_length_array_path = None, box_dimensions_array_path = None, cylinder_radius_array_path = None, cylinder_height_array_path = None, mask_shape = None, observer = None) -> int:
  """
  GenerateMaskFromSimpleShapes

  :param data_container_array | DataContainerArray
  :param mask_array_path | DataArrayPath MaskArrayPath
  :param centers_array_path | DataArrayPath CentersArrayPath
  :param axes_length_array_path | DataArrayPath AxesLengthArrayPath
  :param box_dimensions_array_path | DataArrayPath BoxDimensionsArrayPath
  :param cylinder_radius_array_path | DataArrayPath CylinderRadiusArrayPath
  :param cylinder_height_array_path | DataArrayPath CylinderHeightArrayPath
  :param mask_shape | int MaskShape
  :param observer | Observer
  """

  filter = dream3dreview.GenerateMaskFromSimpleShapes()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if mask_array_path is not None:
    filter.MaskArrayPath = mask_array_path
  if centers_array_path is not None:
    filter.CentersArrayPath = centers_array_path
  if axes_length_array_path is not None:
    filter.AxesLengthArrayPath = axes_length_array_path
  if box_dimensions_array_path is not None:
    filter.BoxDimensionsArrayPath = box_dimensions_array_path
  if cylinder_radius_array_path is not None:
    filter.CylinderRadiusArrayPath = cylinder_radius_array_path
  if cylinder_height_array_path is not None:
    filter.CylinderHeightArrayPath = cylinder_height_array_path
  if mask_shape is not None:
    filter.MaskShape = mask_shape
  filter.execute()
  return filter.ErrorCode

def create_arrayof_indices(data_container_array, index_array_path = None, observer = None) -> int:
  """
  CreateArrayofIndices

  :param data_container_array | DataContainerArray
  :param index_array_path | DataArrayPath IndexArrayPath
  :param observer | Observer
  """

  filter = dream3dreview.CreateArrayofIndices()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if index_array_path is not None:
    filter.IndexArrayPath = index_array_path
  filter.execute()
  return filter.ErrorCode

def compute_feature_eigenstrains(data_container_array, poisson_ratio = None, use_ellipsoidal_grains = None, use_correctional_matrix = None, beta11 = None, beta22 = None, beta33 = None, beta23 = None, beta13 = None, beta12 = None, axis_lengths_array_path = None, axis_euler_angles_array_path = None, elastic_strains_array_path = None, eigenstrains_array_name = None, observer = None) -> int:
  """
  ComputeFeatureEigenstrains

  :param data_container_array | DataContainerArray
  :param poisson_ratio | float PoissonRatio
  :param use_ellipsoidal_grains | bool UseEllipsoidalGrains
  :param use_correctional_matrix | bool UseCorrectionalMatrix
  :param beta11 | float Beta11
  :param beta22 | float Beta22
  :param beta33 | float Beta33
  :param beta23 | float Beta23
  :param beta13 | float Beta13
  :param beta12 | float Beta12
  :param axis_lengths_array_path | DataArrayPath AxisLengthsArrayPath
  :param axis_euler_angles_array_path | DataArrayPath AxisEulerAnglesArrayPath
  :param elastic_strains_array_path | DataArrayPath ElasticStrainsArrayPath
  :param eigenstrains_array_name | QString EigenstrainsArrayName
  :param observer | Observer
  """

  filter = dream3dreview.ComputeFeatureEigenstrains()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if poisson_ratio is not None:
    filter.PoissonRatio = poisson_ratio
  if use_ellipsoidal_grains is not None:
    filter.UseEllipsoidalGrains = use_ellipsoidal_grains
  if use_correctional_matrix is not None:
    filter.UseCorrectionalMatrix = use_correctional_matrix
  if beta11 is not None:
    filter.Beta11 = beta11
  if beta22 is not None:
    filter.Beta22 = beta22
  if beta33 is not None:
    filter.Beta33 = beta33
  if beta23 is not None:
    filter.Beta23 = beta23
  if beta13 is not None:
    filter.Beta13 = beta13
  if beta12 is not None:
    filter.Beta12 = beta12
  if axis_lengths_array_path is not None:
    filter.AxisLengthsArrayPath = axis_lengths_array_path
  if axis_euler_angles_array_path is not None:
    filter.AxisEulerAnglesArrayPath = axis_euler_angles_array_path
  if elastic_strains_array_path is not None:
    filter.ElasticStrainsArrayPath = elastic_strains_array_path
  if eigenstrains_array_name is not None:
    filter.EigenstrainsArrayName = eigenstrains_array_name
  filter.execute()
  return filter.ErrorCode

def adaptive_alignment_feature(data_container_array, good_voxels_array_path = None, observer = None) -> int:
  """
  AdaptiveAlignmentFeature

  :param data_container_array | DataContainerArray
  :param good_voxels_array_path | DataArrayPath GoodVoxelsArrayPath
  :param observer | Observer
  """

  filter = dream3dreview.AdaptiveAlignmentFeature()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if good_voxels_array_path is not None:
    filter.GoodVoxelsArrayPath = good_voxels_array_path
  filter.execute()
  return filter.ErrorCode

def adaptive_alignment_misorientation(data_container_array, misorientation_tolerance = None, use_good_voxels = None, quats_array_path = None, cell_phases_array_path = None, good_voxels_array_path = None, crystal_structures_array_path = None, observer = None) -> int:
  """
  AdaptiveAlignmentMisorientation

  :param data_container_array | DataContainerArray
  :param misorientation_tolerance | float MisorientationTolerance
  :param use_good_voxels | bool UseGoodVoxels
  :param quats_array_path | DataArrayPath QuatsArrayPath
  :param cell_phases_array_path | DataArrayPath CellPhasesArrayPath
  :param good_voxels_array_path | DataArrayPath GoodVoxelsArrayPath
  :param crystal_structures_array_path | DataArrayPath CrystalStructuresArrayPath
  :param observer | Observer
  """

  filter = dream3dreview.AdaptiveAlignmentMisorientation()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if misorientation_tolerance is not None:
    filter.MisorientationTolerance = misorientation_tolerance
  if use_good_voxels is not None:
    filter.UseGoodVoxels = use_good_voxels
  if quats_array_path is not None:
    filter.QuatsArrayPath = quats_array_path
  if cell_phases_array_path is not None:
    filter.CellPhasesArrayPath = cell_phases_array_path
  if good_voxels_array_path is not None:
    filter.GoodVoxelsArrayPath = good_voxels_array_path
  if crystal_structures_array_path is not None:
    filter.CrystalStructuresArrayPath = crystal_structures_array_path
  filter.execute()
  return filter.ErrorCode

def adaptive_alignment_mutual_information(data_container_array, misorientation_tolerance = None, use_good_voxels = None, quats_array_path = None, cell_phases_array_path = None, good_voxels_array_path = None, crystal_structures_array_path = None, global_correction = None, image_data_array_path = None, observer = None) -> int:
  """
  AdaptiveAlignmentMutualInformation

  :param data_container_array | DataContainerArray
  :param misorientation_tolerance | float MisorientationTolerance
  :param use_good_voxels | bool UseGoodVoxels
  :param quats_array_path | DataArrayPath QuatsArrayPath
  :param cell_phases_array_path | DataArrayPath CellPhasesArrayPath
  :param good_voxels_array_path | DataArrayPath GoodVoxelsArrayPath
  :param crystal_structures_array_path | DataArrayPath CrystalStructuresArrayPath
  :param global_correction | int GlobalCorrection
  :param image_data_array_path | DataArrayPath ImageDataArrayPath
  :param observer | Observer
  """

  filter = dream3dreview.AdaptiveAlignmentMutualInformation()
  filter.setDataContainerArray(data_container_array)
  if observer is not None:
    filter.connectObserver(observer)
  if misorientation_tolerance is not None:
    filter.MisorientationTolerance = misorientation_tolerance
  if use_good_voxels is not None:
    filter.UseGoodVoxels = use_good_voxels
  if quats_array_path is not None:
    filter.QuatsArrayPath = quats_array_path
  if cell_phases_array_path is not None:
    filter.CellPhasesArrayPath = cell_phases_array_path
  if good_voxels_array_path is not None:
    filter.GoodVoxelsArrayPath = good_voxels_array_path
  if crystal_structures_array_path is not None:
    filter.CrystalStructuresArrayPath = crystal_structures_array_path
  if global_correction is not None:
    filter.GlobalCorrection = global_correction
  if image_data_array_path is not None:
    filter.ImageDataArrayPath = image_data_array_path
  filter.execute()
  return filter.ErrorCode

